#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#include "uart.h"


/* define CPU frequency in Mhz here if not defined in Makefile */
#ifndef F_CPU
#define F_CPU 16000000UL
#endif

#include <util/delay.h> 

#define UART_BAUD_RATE      19200

int input_interrupt_counter=0;
uint16_t input_interrupt_prescaler=0;
uint16_t prescale_input_interrupt=256;
uint8_t enable_input_interrupt_counter=0;

uint8_t in_ww_count_mode=0;
int ww_counter=0;

uint16_t ReadChannel(uint8_t mux)
{
  uint8_t i;
  uint16_t result;
 
  ADMUX = mux;                      // Kanal waehlen
  ADMUX |= (0<<REFS1) | (1<<REFS0); // AVCC nutzen
 
  ADCSRA = (1<<ADEN) | (1<<ADPS1) | (1<<ADPS0);    // Frequenzvorteiler 
                               // setzen auf 8 (1) und ADC aktivieren (1)
 
  /* nach Aktivieren des ADC wird ein "Dummy-Readout" empfohlen, man liest
     also einen Wert und verwirft diesen, um den ADC "warmlaufen zu lassen" */
  ADCSRA |= (1<<ADSC);              // eine ADC-Wandlung 
  while ( ADCSRA & (1<<ADSC) ) {
     ;     // auf Abschluss der Konvertierung warten 
  }
  result = ADCW;  // ADCW muss einmal gelesen werden,
                  // sonst wird Ergebnis der nchsten Wandlung
                  // nicht bernommen.
 
  /* Eigentliche Messung - Mittelwert aus 4 aufeinanderfolgenden Wandlungen */
  result = 0; 
  for( i=0; i<4; i++ )
  {
    ADCSRA |= (1<<ADSC);            // eine Wandlung "single conversion"
    while ( ADCSRA & (1<<ADSC) ) {
      ;   // auf Abschluss der Konvertierung warten
    }
    result += ADCW;		    // Wandlungsergebnisse aufaddieren
  }
  ADCSRA &= ~(1<<ADEN);             // ADC deaktivieren (2)
 
  result /= 4;                     // Summe durch vier teilen = arithm. Mittelwert
 
  return result;
}

SIGNAL(SIG_INTERRUPT0)     
{
	if(enable_input_interrupt_counter==1)
		if (input_interrupt_prescaler==0)
		{
			input_interrupt_counter++;
			input_interrupt_prescaler=prescale_input_interrupt;
		}

}

ISR( TIMER0_OVF_vect )                       
{

	TCNT0 = (uint8_t)(int16_t)-(F_CPU / 1024 * 10e-3 + 0.5);

	if (!(PIND & (1<<PIND3)))
	{
		if(in_ww_count_mode==1)
		{
			ww_counter++;
		}
		else
		{
			in_ww_count_mode=1;
			ww_counter=0;
		}
	}
	else
	{
		if(in_ww_count_mode==1)
			in_ww_count_mode=0;
	}

}

int main(void)
{
    unsigned int c;
    char buffer[7];
	uint16_t adcval;
 
 	DDRD  = 0b00110000;           
    PORTD = 0b00110000; 

	GIMSK = 0b01000000;
	MCUCR = 0b00000011;

	TCCR0 = (1<<CS02)|(1<<CS00);		
  	TIMSK = 1<<TOIE0;	

    uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU) ); 

    sei();
    
    for(;;)
    {

		if(enable_input_interrupt_counter==1)
			if (input_interrupt_prescaler>=1)
				input_interrupt_prescaler--;

        c = uart_getc();
        if ( c & UART_NO_DATA )
        {

        }
        else
        {


//1=ADC lesen
//2=Interrupt-Counter lesen
//3=Interrupt-Counter reset
//4=Interrupt-Counter starten
//5=Interrupt-Counter sperren
//6=Pin3-Status
//7=Check
//8=Pin3 Zeitmessung
//9=Zeitmessung rcksetzen

			if(c==1)
			{
				adcval = ReadChannel(0);
				itoa(adcval,buffer,10);
				uart_puts(buffer);
			}
			else
			if(c==2)
			{
				itoa(input_interrupt_counter, buffer, 10);  
    			uart_puts(buffer);
			}
			else
			if(c==3)
			{
				input_interrupt_counter=0;
			}
			else
			if(c==4)
			{
				enable_input_interrupt_counter=1;
			}
			else
			if(c==5)
			{
				enable_input_interrupt_counter=0;
			}
			else
			if(c==6)
			{
				if (!( PIND & (1<<PIND3)))
					uart_putc(2);
				else
					uart_putc(3);
				
			}
			else
			if(c==7)
			{
				uart_putc(5);
			}
			else
			if(c==8)
			{
				itoa(ww_counter, buffer, 10);  
    			uart_puts(buffer);
			}
			else
			if(c==9)
			{
				ww_counter=0;
			}
		}
	
    }
    
}
