unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, XPMan, ExtCtrls, Menus, ComCtrls, ScktComp, StdCtrls, RS232_MCU;

type
  TForm1 = class(TForm)
    PopupMenu:  TPopupMenu;
    Zeigen1:    TMenuItem;
    Beenden1:   TMenuItem;
    TrayIcon:   TTrayIcon;
    XPManifest: TXPManifest;
    StartupTimer: TTimer;
    StatusBar:  TStatusBar;
    GroupBox1:  TGroupBox;
    LBL_Temp:   TLabel;
    Text3:      TLabel;
    Text1:      TLabel;
    Text2:      TLabel;
    Text4:      TLabel;
    LBL_In:     TLabel;
    LBL_In_Sum: TLabel;
    UpdateTimer: TTimer;
    PNL_WW_Taster: TPanel;
    Label10: TLabel;
    Label12: TLabel;
    LBL_In_Diff: TLabel;
    LBL_WW_Zeit: TLabel;
    procedure UpdateTimerTimer(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Zeigen1Click(Sender: TObject);
    procedure Beenden1Click(Sender: TObject);
    procedure StartupTimerTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure TrayIconClick(Sender: TObject);
    procedure StatusMessage(Text: string);
    procedure BalloonHint(Text: string; IsError: boolean);
    procedure Shutdown;
    procedure ClientSocketConnect(Sender: TObject; Socket: TCustomWinSocket);
    procedure ClientSocketDisconnect(Sender: TObject; Socket: TCustomWinSocket);
    procedure ClientSocketError(Sender: TObject; Socket: TCustomWinSocket;
      ErrorEvent: TErrorEvent; var ErrorCode: integer);
    procedure ClientSocketRead(Sender: TObject; Socket: TCustomWinSocket);
    procedure ClientSocketWrite(Text: string);
    procedure ProcessCommand(Command: string);
  private
    { Private declarations }
  public
    { Public declarations }
    ClientSocket: TClientSocket;
    RS232_MCU:    TRS232_MCU;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

const
  ServerSocketPort: integer = 10020;
  ServerSocketHost: string = 'localhost';
  StartupInterval: integer = 1000;
  ShutdownTime: integer = 1000;
  HintTimeout: integer = 1000;
  HideAfterStartup = True;
  UpdateInterval = 125;
  MaxMCU_InDiff = 20;

var
  BootupFlag:    integer;
  AllowShutdown: boolean;
  MCU_Temp, MCU_In, MCU_In_Sum, MCU_In_Old, MCU_WW_Zeit: integer;
  MCU_WW_Taster:boolean;

type
  TSTringdynarray = array of string;

////////////////////////////////////////////////////////////////////////////////

{$REGION 'Explode & Kommandoverarbeitung'}

function Explode(const Separator, S: string; Limit: integer = 0): TStringDynArray;
var
  SepLen: integer;
  F, P:   PChar;
  ALen, Index: integer;
begin
  SetLength(Result, 0);
  if (S = '') or (Limit < 0) then
    Exit;
  if Separator = '' then
  begin
    SetLength(Result, 1);
    Result[0] := S;
    Exit;
  end;
  SepLen := Length(Separator);
  ALen   := Limit;
  SetLength(Result, ALen);

  Index := 0;
  P     := PChar(S);
  while P^ <> #0 do
  begin
    F := P;
    P := AnsiStrPos(P, PChar(Separator));
    if (P = nil) or ((Limit > 0) and (Index = Limit - 1)) then
      P := StrEnd(F);
    if Index >= ALen then
    begin
      Inc(ALen, 5);
      SetLength(Result, ALen);
    end;
    SetString(Result[Index], F, P - F);
    Inc(Index);
    if P^ <> #0 then
      Inc(P, SepLen);
  end;
  if Index < ALen then
    SetLength(Result, Index);
end;

procedure TForm1.ProcessCommand(Command: string);
var
  Action: string;
  CommandParts: TSTringdynarray;
begin
  CommandParts := Explode(';', Command);
  Action := string(CommandParts[0]);

  if Action = 'GET_TEMP' then
  begin
    ClientSocketWrite('GET_TEMP;' + IntToStr(MCU_Temp));
  end
  else
  if Action = 'GET_IN' then
  begin
    ClientSocketWrite('GET_IN;' + IntToStr(MCU_In));
  end
  else
  if Action = 'GET_IN_SUM' then
  begin
    ClientSocketWrite('GET_IN_SUM;' + IntToStr(MCU_In_Sum));
  end
  else
  if Action = 'RESET_IN' then
  begin
    RS232_MCU.ResetCounter;
    ClientSocketWrite('RESET_IN;OK');
  end
  else
  if Action = 'ENABLE_IN' then
  begin
    RS232_MCU.EnableCounter;
    ClientSocketWrite('ENABLE_IN;OK');
  end
  else
  if Action = 'DISABLE_IN' then
  begin
    RS232_MCU.DisableCounter;
    ClientSocketWrite('DISABLE_IN;OK');
  end
  else
  if Action = 'GET_WW_TIME' then
  begin
    ClientSocketWrite('GET_WW_TIME;'+IntToStr(MCU_WW_Zeit));
  end
  else
  if Action = 'GET_WW_STATE' then
  begin
    if MCU_WW_Taster then
      ClientSocketWrite('GET_WW_STATE;1')
    else
      ClientSocketWrite('GET_WW_STATE;0');
  end
  else
  if Action = 'RESET_WW_TIME' then
  begin
    RS232_MCU.ResetPortOnTime;
    ClientSocketWrite('RESET_WW_TIME;OK');
  end
  else

  if Action = 'LOCK' then
  begin
    if CommandParts[1] = '1' then
      AllowShutdown := False
    else
      AllowShutdown := True;
    ClientSocketWrite('LOCK;OK');
  end
  else
  if Action = 'CHECK' then
  begin
    if RS232_MCU.Check then
      ClientSocketWrite('CHECK;OK')
    else
      ClientSocketWrite('CHECK;ERROR');
  end
  else
  if Action = 'EXIT' then
  begin
    ClientSocketWrite('EXIT;OK');
    Shutdown;
  end;

  StatusMessage('Bereit');
end;

{$ENDREGION}

////////////////////////////////////////////////////////////////////////////////

{$REGION 'Statusnachrichten & Shutdown'}

procedure TForm1.StatusMessage(Text: string);
begin
  StatusBar.Panels.Items[0].Text := Text;
  StatusBar.Repaint;
end;

procedure TForm1.BalloonHint(Text: string; IsError: boolean);
begin
  TrayIcon.BalloonHint := Text;
  if IsError then
    TrayIcon.BalloonFlags := bfError
  else
    TrayIcon.BalloonFlags := bfInfo;
  TrayIcon.BalloonTimeout := HintTimeout;
  TrayIcon.ShowBalloonHint;
end;

procedure TForm1.Shutdown;
begin
  StatusMessage('Beende Programm');

  UpdateTimer.Enabled := False;

  RS232_MCU.ResetCounter;
  Sleep(ShutdownTime);
  RS232_MCU.Disconnect;

  if (ClientSocket.Active) then
  begin
    ClientSocketWrite('EXIT');
    ClientSocket.Close;
  end;

  Halt;
end;

{$ENDREGION}

////////////////////////////////////////////////////////////////////////////////

{$REGION 'Socket'}

procedure TForm1.ClientSocketConnect(Sender: TObject; Socket: TCustomWinSocket);
begin
  ClientSocketWrite('WELCOME');
  BalloonHint('Verbunden mit ' + ServerSocketHost + ':' +
    IntToStr(ServerSocketPort), False);
end;

procedure TForm1.ClientSocketDisconnect(Sender: TObject; Socket: TCustomWinSocket);
begin
  AllowShutdown := True;
  ClientSocket.Active := False;
  BalloonHint('Verbindung verloren! Beende Programm', True);
  Shutdown;
end;

procedure TForm1.ClientSocketError(Sender: TObject; Socket: TCustomWinSocket;
  ErrorEvent: TErrorEvent; var ErrorCode: integer);
begin
  AllowShutdown := True;
  ClientSocket.Active := False;
  BalloonHint('Fehler ' + IntToStr(ErrorCode) + '. Beende Programm', True);
  Shutdown;
  ErrorCode := 0;
end;

procedure TForm1.ClientSocketRead(Sender: TObject; Socket: TCustomWinSocket);
var
  Command: string;
  I: integer;
  CommandList: TSTringdynarray;
begin

  Command := Socket.ReceiveText;

  CommandList := Explode(':', Command);

  for I := 0 to length(CommandList) - 1 do
  begin
    Command := CommandList[I];
    if Command <> '' then
      ProcessCommand(Command);
  end;

end;

procedure TForm1.ClientSocketWrite(Text: string);
begin
  if (ClientSocket.Active) then
    ClientSocket.Socket.SendText(':' + Text)
  else
  begin
    AllowShutdown := True;
    ClientSocket.Active := False;
    BalloonHint('Verbindung verloren! Beende Programm', True);
    Shutdown;
  end;
end;

{$ENDREGION}

////////////////////////////////////////////////////////////////////////////////

{$REGION 'GUI'}
procedure TForm1.Beenden1Click(Sender: TObject);
begin
  if AllowShutdown then
    Shutdown
  else
    BalloonHint('Programm kann nicht beendet werden', True);
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Shutdown;
end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: boolean);
begin
  if AllowShutdown then
    Shutdown
  else
  begin
    CanClose := False;
    BalloonHint('Programm kann nicht beendet werden', True);
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  StatusMessage('Verbinde mit Server');

  ClientSocket      := TClientSocket.Create(Form1);
  ClientSocket.OnConnect := ClientSocketConnect;
  ClientSocket.OnDisconnect := ClientSocketDisconnect;
  ClientSocket.OnError := ClientSocketError;
  ClientSocket.OnRead := ClientSocketRead;
  ClientSocket.Port := ServerSocketPort;
  ClientSocket.Host := ServerSocketHost;
  ClientSocket.Active := True;

  StatusMessage('Starte MCU');
  BootupFlag    := 0;
  AllowShutdown := True;
  StartupTimer.Interval := StartupInterval;
  StartupTimer.Enabled := True;
end;

procedure TForm1.StartupTimerTimer(Sender: TObject);
begin
  if BootupFlag = 0 then
  begin
    RS232_MCU.Connect;
    if RS232_MCU.Check = False then
    begin
      StartupTimer.Enabled := False;
      ClientSocketWrite('ERROR');
      StatusMessage('MCU antwortet nicht');
      BalloonHint('MCU antwortet nicht', True);
    end
    else
    begin
      BootupFlag := 1;
    end;
  end
  else if BootupFlag = 1 then
  begin
    UpdateTimer.Interval := UpdateInterval;
    UpdateTimer.Enabled  := True;
    ClientSocketWrite('READY');
    StatusMessage('Bereit');
    BalloonHint('Client bereit', False);
    BootupFlag := 2;
  end
  else
  begin
    StartupTimer.Enabled := False;
    Visible := not HideAfterStartup;
  end;
end;

procedure TForm1.TrayIconClick(Sender: TObject);
begin
  Form1.Visible := not Form1.Visible;
end;

procedure TForm1.Zeigen1Click(Sender: TObject);
begin
  Form1.Visible := not Form1.Visible;
end;

{$ENDREGION}

////////////////////////////////////////////////////////////////////////////////

{$REGION 'Update'}

procedure TForm1.UpdateTimerTimer(Sender: TObject);
var
  MCU_In_Diff: integer;
begin

  StatusMessage('Update luft');

  MCU_Temp := RS232_MCU.ReadADC;
  MCU_In   := RS232_MCU.ReadCounter;

  MCU_In_Diff := MCU_In - MCU_In_Old;

  LBL_In_Diff.Caption := FloatToStrF(MCU_In_Diff, ffNumber, 15, 2);

  if MCU_In_Diff > MaxMCU_InDiff then
  begin
    MCU_In      := MCU_In_Old;
    MCU_In_Diff := MCU_In - MCU_In_Old;
    RS232_MCU.ResetCounter;
    MCU_In_Old := 0;
  end;

  if MCU_In_Diff > 0 then
    MCU_In_Sum := MCU_In_Sum + MCU_In_Diff;

  MCU_In_Old := MCU_In;

  LBL_Temp.Caption   := IntToStr(MCU_Temp);
  LBL_In.Caption     := IntToStr(MCU_In);
  LBL_In_Sum.Caption := IntToStr(MCU_In_Sum);

  MCU_WW_Taster:=RS232_MCU.ReadPortState;

  if MCU_WW_Taster then
    PNL_WW_Taster.Color:=clLime
  else
    PNL_WW_Taster.Color:=clRed;

  MCU_WW_Zeit:=RS232_MCU.ReadPortOnTime;

  LBL_WW_Zeit.Caption := IntToStr(MCU_WW_Zeit);

  StatusMessage('Bereit');

end;

{$ENDREGION}

////////////////////////////////////////////////////////////////////////////////


end.
