unit RS232_MCU;

interface

type
  TRS232_MCU = class
    procedure Connect;
    procedure Disconnect;
    function ReadADC: integer;
    function ReadCounter: integer;
    procedure ResetCounter;
    procedure EnableCounter;
    procedure DisableCounter;
    function ReadPortState: boolean;
    function Check: boolean;
    function ReadPortOnTime: integer;
    procedure ResetPortOnTime;
  end;

implementation

uses RS232, SysUtils;

var
  ComOpen: boolean;

 //1=ADC lesen
 //2=Interrupt-Counter lesen
 //3=Interrupt-Counter reset
 //4=Interrupt-Counter starten
 //5=Interrupt-Counter sperren
 //6=Pin3-Status
 //7=Check
 //8=Pin3 Zeitmessung
 //9=Zeitmessung rcksetzen

////////////////////////////////////////////////////////////////////////////////

{$REGION 'MCU lesen'}

function ReadMCU(Port: word): integer;
var
  Read:  boolean;
  Buffer, Counter: integer;
  Value: string;
begin

  Read    := True;
  Counter := 0;
  Value   := '';

  SendByte(Port);

  while (Read) do
  begin
    Buffer := ReadByte;

    if (Buffer > 47) and (Buffer < 58) then
      Value := Value + chr(buffer)
    else
      Read  := False;

    Counter := Counter + 1;
    if Counter > 4 then
      Read := False;
  end;

  if length(Value) = 0 then
    ReadMCU := 0
  else
    ReadMCU := StrToInt(Value);

end;

{$ENDREGION}

////////////////////////////////////////////////////////////////////////////////


{$REGION 'Initialisierung'}

procedure TRS232_MCU.Connect;
begin
  if ComOpen = False then
  begin
    OPENCOM('COM1: baud=19200 data=8 parity=N stop=1');
    ComOpen := True;
  end;
end;

procedure TRS232_MCU.Disconnect;
begin
  if ComOpen then
  begin
    CLOSECOM;
    ComOpen := False;
  end;
end;

{$ENDREGION}

////////////////////////////////////////////////////////////////////////////////

{$REGION 'MCU-Funktionen'}

function TRS232_MCU.ReadADC: integer;
begin
  ReadADC := ReadMCU(1);
end;

function TRS232_MCU.ReadCounter: integer;
begin
  ReadCounter := ReadMCU(2);
end;

procedure TRS232_MCU.ResetCounter;
begin
  SendByte(3);
end;

procedure TRS232_MCU.EnableCounter;
begin
  SendByte(4);
end;

procedure TRS232_MCU.DisableCounter;
begin
  SendByte(5);
end;

function TRS232_MCU.ReadPortState: boolean;
var
  Buffer: integer;
begin
  SendByte(6);
  Buffer := ReadByte;
  if Buffer = 2 then
    ReadPortState := True
  else
    ReadPortState := False;
end;

function TRS232_MCU.Check: boolean;
var
  Buffer: integer;
begin
  SendByte(7);
  Buffer := ReadByte;
  if Buffer = 5 then
    Check := True
  else
    Check := False;
end;

function TRS232_MCU.ReadPortOnTime: integer;
begin
  ReadPortOnTime := ReadMCU(8);
end;

procedure TRS232_MCU.ResetPortOnTime;
begin
  SendByte(9);
end;

{$ENDREGION}

////////////////////////////////////////////////////////////////////////////////

end.
