unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, XPMan, ComCtrls, StdCtrls, ScktComp, Menus, RS232_Relaisboard;

type
  TForm1 = class(TForm)
    StatusBar:   TStatusBar;
    XPManifest:  TXPManifest;
    StartupTimer: TTimer;
    Board1:      TGroupBox;
    Board1_Rel1: TPanel;
    Board1_Rel2: TPanel;
    Board1_Rel3: TPanel;
    Board1_Rel4: TPanel;
    Board1_Rel5: TPanel;
    Board1_Rel6: TPanel;
    Board1_Rel7: TPanel;
    Board1_Rel8: TPanel;
    Board2:      TGroupBox;
    Board2_Rel1: TPanel;
    Board2_Rel2: TPanel;
    Board2_Rel3: TPanel;
    Board2_Rel4: TPanel;
    Board2_Rel5: TPanel;
    Board2_Rel6: TPanel;
    Board2_Rel7: TPanel;
    Board2_Rel8: TPanel;
    TrayIcon:    TTrayIcon;
    PopupMenu:   TPopupMenu;
    Zeigen1:     TMenuItem;
    Beenden1:    TMenuItem;
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Zeigen1Click(Sender: TObject);
    procedure Beenden1Click(Sender: TObject);
    procedure StartupTimerTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure TrayIconClick(Sender: TObject);
    procedure StatusMessage(Text: string);
    procedure BalloonHint(Text: string; IsError: boolean);
    procedure Shutdown;
    procedure ClientSocketConnect(Sender: TObject; Socket: TCustomWinSocket);
    procedure ClientSocketDisconnect(Sender: TObject; Socket: TCustomWinSocket);
    procedure ClientSocketError(Sender: TObject; Socket: TCustomWinSocket;
      ErrorEvent: TErrorEvent; var ErrorCode: integer);
    procedure ClientSocketRead(Sender: TObject; Socket: TCustomWinSocket);
    procedure ClientSocketWrite(Text: string);
    procedure ProcessCommand(Command: string);
    procedure SetRelaisGUIState(Board, Relais, State: integer);
    procedure SetRelaisGUIStateZerobyte;
  private
    { Private declarations }
  public
    { Public declarations }
    ClientSocket:      TClientSocket;
    RS232_Relaisboard: TRS232_Relaisboard;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

const
  ServerSocketPort: integer = 10021;
  ServerSocketHost: string = 'localhost';
  StartupInterval: integer = 1000;
  ShutdownTime: integer = 1000;
  HintTimeout: integer = 1000;
  HideAfterStartup = True;

var
  BootupFlag:    integer;
  AllowShutdown: boolean;

type
  TSTringdynarray = array of string;

////////////////////////////////////////////////////////////////////////////////

{$REGION 'Explode & Kommandoverarbeitung'}

function Explode(const Separator, S: string; Limit: integer = 0): TStringDynArray;
var
  SepLen: integer;
  F, P:   PChar;
  ALen, Index: integer;
begin
  SetLength(Result, 0);
  if (S = '') or (Limit < 0) then
    Exit;
  if Separator = '' then
  begin
    SetLength(Result, 1);
    Result[0] := S;
    Exit;
  end;
  SepLen := Length(Separator);
  ALen   := Limit;
  SetLength(Result, ALen);

  Index := 0;
  P     := PChar(S);
  while P^ <> #0 do
  begin
    F := P;
    P := AnsiStrPos(P, PChar(Separator));
    if (P = nil) or ((Limit > 0) and (Index = Limit - 1)) then
      P := StrEnd(F);
    if Index >= ALen then
    begin
      Inc(ALen, 5);
      SetLength(Result, ALen);
    end;
    SetString(Result[Index], F, P - F);
    Inc(Index);
    if P^ <> #0 then
      Inc(P, SepLen);
  end;
  if Index < ALen then
    SetLength(Result, Index);
end;

procedure TForm1.ProcessCommand(Command: string);
var
  Action: string;
  CommandParts: TSTringdynarray;
  Board, Relais, Status: integer;
begin
  CommandParts := Explode(';', Command);
  Action := string(CommandParts[0]);

  if Action = 'SET_RELAIS_DUMMY' then
  begin
    Board  := StrToInt(string(CommandParts[1]));
    Relais := StrToInt(string(CommandParts[2]));
    Status := StrToInt(string(CommandParts[3]));

    RS232_Relaisboard.SetRelaisDummy(board, relais, status);
    ClientSocketWrite('SET_RELAIS_DUMMY;OK');

  end
  else
  if Action = 'SET_RELAIS' then
  begin
    Board  := StrToInt(string(CommandParts[1]));
    Relais := StrToInt(string(CommandParts[2]));
    Status := StrToInt(string(CommandParts[3]));

    if RS232_Relaisboard.SetRelais(board, relais, status) then
      ClientSocketWrite('SET_RELAIS;OK')
    else
      ClientSocketWrite('SET_RELAIS;ERROR');

  end
  else

  if Action = 'RESUME_DUMMY' then
  begin
    Board := StrToInt(string(CommandParts[1]));
    if RS232_Relaisboard.ResumeDummySettings(board) then
      ClientSocketWrite('RESUME_DUMMY;OK')
    else
      ClientSocketWrite('RESUME_DUMMY;ERROR');
  end
  else

  if Action = 'ZEROBYTE' then
  begin
    Board := StrToInt(string(CommandParts[1]));
    if RS232_Relaisboard.Zerobyte(Board) then
      ClientSocketWrite('ZEROBYTE;OK')
    else
      ClientSocketWrite('ZEROBYTE;ERROR');
  end
  else

  if Action = 'LOCK' then
  begin
    if CommandParts[1] = '1' then
      AllowShutdown := False
    else
      AllowShutdown := True;
    ClientSocketWrite('LOCK;OK');
  end
  else
  if Action = 'CHECK' then
  begin
    if RS232_Relaisboard.NOPCheck then
      ClientSocketWrite('CHECK;OK')
    else
      ClientSocketWrite('CHECK;ERROR');
  end
  else
  if Action = 'EXIT' then
  begin
    ClientSocketWrite('EXIT;OK');
    Shutdown;
  end;

  StatusMessage('Bereit');
end;

{$ENDREGION}

////////////////////////////////////////////////////////////////////////////////

{$REGION 'Statusnachrichten & Shutdown'}

procedure TForm1.StatusMessage(Text: string);
begin
  StatusBar.Panels.Items[0].Text := Text;
  StatusBar.Repaint;
end;

procedure TForm1.BalloonHint(Text: string; IsError: boolean);
begin
  TrayIcon.BalloonHint := Text;
  if IsError then
    TrayIcon.BalloonFlags := bfError
  else
    TrayIcon.BalloonFlags := bfInfo;
  TrayIcon.BalloonTimeout := HintTimeout;
  TrayIcon.ShowBalloonHint;
end;

procedure TForm1.Shutdown;
begin
  StatusMessage('Beende Programm');

  RS232_Relaisboard.Zerobyte(1);
  RS232_Relaisboard.Zerobyte(2);
  Sleep(ShutdownTime);
  RS232_Relaisboard.Disconnect;

  if (ClientSocket.Active) then
  begin
    ClientSocketWrite('EXIT');
    ClientSocket.Close;
  end;

  Halt;
end;

{$ENDREGION}

////////////////////////////////////////////////////////////////////////////////

{$REGION 'Socket'}

procedure TForm1.ClientSocketConnect(Sender: TObject; Socket: TCustomWinSocket);
begin
  ClientSocketWrite('WELCOME');
  BalloonHint('Verbunden mit ' + ServerSocketHost + ':' +
    IntToStr(ServerSocketPort), False);
end;

procedure TForm1.ClientSocketDisconnect(Sender: TObject; Socket: TCustomWinSocket);
begin
  AllowShutdown := True;
  ClientSocket.Active := False;
  BalloonHint('Verbindung verloren! Beende Programm', True);
  Shutdown;
end;

procedure TForm1.ClientSocketError(Sender: TObject; Socket: TCustomWinSocket;
  ErrorEvent: TErrorEvent; var ErrorCode: integer);
begin
  AllowShutdown := True;
  ClientSocket.Active := False;
  BalloonHint('Fehler ' + IntToStr(ErrorCode) + '. Beende Programm', True);
  Shutdown;
  ErrorCode := 0;
end;

procedure TForm1.ClientSocketRead(Sender: TObject; Socket: TCustomWinSocket);
var
  Command: string;
  I: integer;
  CommandList: TSTringdynarray;
begin

  Command := Socket.ReceiveText;

  CommandList := Explode(':', Command);

  for I := 0 to length(CommandList) - 1 do
  begin
    Command := CommandList[I];
    if Command <> '' then
      ProcessCommand(Command);
  end;

end;

procedure TForm1.ClientSocketWrite(Text: string);
begin
  if (ClientSocket.Active) then
    ClientSocket.Socket.SendText(':' + Text)
  else
  begin
    AllowShutdown := True;
    ClientSocket.Active := False;
    BalloonHint('Verbindung verloren! Beende Programm', True);
    Shutdown;
  end;
end;

{$ENDREGION}

////////////////////////////////////////////////////////////////////////////////

{$REGION 'GUI'}

procedure TForm1.Beenden1Click(Sender: TObject);
begin
  if AllowShutdown then
    Shutdown
  else
    BalloonHint('Programm kann nicht beendet werden', True);
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Shutdown;
end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: boolean);
begin
  if AllowShutdown then
    Shutdown
  else
  begin
    CanClose := False;
    BalloonHint('Programm kann nicht beendet werden', True);
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  StatusMessage('Verbinde mit Server');

  ClientSocket      := TClientSocket.Create(Form1);
  ClientSocket.OnConnect := ClientSocketConnect;
  ClientSocket.OnDisconnect := ClientSocketDisconnect;
  ClientSocket.OnError := ClientSocketError;
  ClientSocket.OnRead := ClientSocketRead;
  ClientSocket.Port := ServerSocketPort;
  ClientSocket.Host := ServerSocketHost;
  ClientSocket.Active := True;

  StatusMessage('Starte Relaisboard');
  BootupFlag    := 0;
  AllowShutdown := True;
  StartupTimer.Interval := StartupInterval;
  StartupTimer.Enabled := True;
end;

procedure TForm1.StartupTimerTimer(Sender: TObject);
begin
  if BootupFlag = 0 then
  begin
    RS232_Relaisboard.Connect;
    if RS232_Relaisboard.InitBoards = False then
    begin
      StartupTimer.Enabled := False;
      ClientSocketWrite('ERROR');
      StatusMessage('Relaisboard nicht erkannt');
      BalloonHint('Relaisboard nicht erkannt', True);
    end
    else
    begin
      BootupFlag := 1;
    end;
  end
  else if BootupFlag = 1 then
  begin
    ClientSocketWrite('READY');
    StatusMessage('Bereit');
    BalloonHint('Client bereit', False);
    BootupFlag := 2;
  end
  else
  begin
    StartupTimer.Enabled := False;
    Visible := not HideAfterStartup;
  end;
end;

procedure TForm1.TrayIconClick(Sender: TObject);
begin
  Form1.Visible := not Form1.Visible;
end;

procedure TForm1.Zeigen1Click(Sender: TObject);
begin
  Form1.Visible := not Form1.Visible;
end;

procedure TForm1.SetRelaisGUIState(Board, Relais, State: integer);
var
  NewColor: TColor;
begin

  NewColor := clBtnFace;

  if State = 1 then
    NewColor := clLime
  else if State = 0 then
    NewColor := clRed
  else if State = 2 then
    NewColor := clMaroon
  else if State = 3 then
    NewColor := clOlive;

  (FindComponent('Board' + IntToStr(Board) + '_Rel' + IntToStr(Relais))
    as TPanel).Color :=
    NewColor;
  (FindComponent('Board' + IntToStr(Board) + '_Rel' + IntToStr(Relais))
    as TPanel).Repaint;

end;

procedure TForm1.SetRelaisGUIStateZerobyte;
var
  I, II: integer;
begin
  for I := 1 to 2 do
    for II := 1 to 8 do
    begin
      (FindComponent('Board' + IntToStr(I) + '_Rel' + IntToStr(II)) as TPanel).Color :=
        clRed;
      (FindComponent('Board' + IntToStr(I) + '_Rel' + IntToStr(II)) as TPanel).Repaint;
    end;
end;

{$ENDREGION}

////////////////////////////////////////////////////////////////////////////////


end.
