unit Programm;

interface

type
  TProgramm = class
    function IsProgrammRunning: boolean;
    procedure StartProgramm;
    procedure StopProgramm;
    procedure InitProgramm;
    procedure NextStep;
  end;

implementation

uses Main, SysUtils;

const
  ProgrammSteps: integer = 46;

var
  ProgrammRunning: boolean;
  CurrentStep:     integer;

////////////////////////////////////////////////////////////////////////////////

{$REGION 'Private Funktionen'}

procedure PrepareNextStep(Interval: integer; StartStep: boolean = True);
begin
  Main.Form1.ProgrammTimer.Interval := Interval;
  Main.Form1.ProgrammTimer.Enabled  := StartStep;
end;

procedure Info(Text: string);
begin
  Main.Form1.AddInfoEntry(Text);
end;

{$ENDREGION}

////////////////////////////////////////////////////////////////////////////////

{$REGION 'Programm Start/Stop/Init'}

function TProgramm.IsProgrammRunning: boolean;
begin
  IsProgrammRunning := ProgrammRunning;
end;

procedure TProgramm.StartProgramm;
begin
  ProgrammRunning := True;
  Main.Form1.ProgrammText3.Caption := 'Programm luft';
  Main.Form1.AddTipEntry('Programm gestartet');
  Main.Form1.ProgrammProgressBar.Position := 0;
  CurrentStep := StrToInt(Main.Form1.Startab.Text);
  PrepareNextStep(1000,true);
end;

procedure TProgramm.StopProgramm;
begin
  ProgrammRunning := False;
  Main.Form1.ProgrammProgressBar.Position := 0;
  Main.Form1.ProgrammText3.Caption := 'Beendet';
  Main.Form1.AddTipEntry('Programm gestopt');
  Main.Form1.ResetMachine;
end;

procedure TProgramm.InitProgramm;
begin
  Main.Form1.ProgrammText1.Caption   := 'Normal 65C';
  Main.Form1.ProgrammText2.Caption   := 'ca. 19,5l, Laufzeit 90 Minuten';
  Main.Form1.ProgrammText3.Caption   := '';
  Main.Form1.ProgrammProgressBar.Max := ProgrammSteps + 1;
  Main.Form1.ProgrammProgressBar.Position := 0;
end;

{$ENDREGION}

////////////////////////////////////////////////////////////////////////////////

{$REGION 'Programmschritte'}
 //WW:
 //1=Sync
 //2=OK
 //3=OKG
 //4=UK

procedure TProgramm.NextStep;
begin
  if ProgrammRunning then
  begin
    CurrentStep := CurrentStep + 1;
    Main.Form1.ProgrammProgressBar.Position := CurrentStep;
    Main.Form1.Startab.Text:=IntToStr(CurrentStep-1);
    
    case CurrentStep of
      1:
      begin
        Info('Programm gestartet');
        Info('Restwasser abpumpen');
        PrepareNextStep(10, False);
        Main.Form1.Ausleeren(10000, True);
      end;
      2:
      begin
        Info('Wasserweiche synchronisieren');
        PrepareNextStep(10, False);
        Main.Form1.Korbwahl(1, True);
      end;
      3:
      begin
        Info('Wasserweiche einstellen: OK');
        PrepareNextStep(10, False);
        Main.Form1.Korbwahl(2, True);
      end;


      4:
      begin
        Info('### Vorsplen ###');
        Info('Fllen 4 Liter');
        PrepareNextStep(10, False);
        Main.Form1.Fuellen(4, True);
      end;
      5:
      begin
        Info('Starte Wasserweichen-Automatik (alle 70 Sekunden wechseln)');
        PrepareNextStep(10, True);
        Main.Form1.AutoWWStart(70000);
      end;
      6:
      begin
        Info('Heizen 35, Umwlzen, warten bis warm');
        Main.Form1.HeizenStart(35, True, True);
      end;
      7:
      begin
        Info('Warte weitere 240 Sekunden auf Vorsplen-Ende');
        PrepareNextStep(240000, True);
      end;
      8:
      begin
        Info('Heizen, Wasserweichen-Automatik & Umwlzen Stop');
        Main.Form1.HeizenStop;
        Main.Form1.AutoWWStop;
        Main.Form1.Machine.Machine_Umwaelzen(False);
        PrepareNextStep(5000, True);
      end;
      9:
      begin
        Info('Wasser abpumpen, Vorsplen fertig');
        PrepareNextStep(10, False);
        Main.Form1.Ausleeren(0, True);
      end;


      10:
      begin
        Info('Wasserweiche einstellen: OK');
        PrepareNextStep(10, True);
        Main.Form1.Korbwahl(2, False);
      end;


      11:
      begin
        Info('### Reinigen ###');
        Info('Fllen 4,5 Liter');
        PrepareNextStep(10, False);
        Main.Form1.Fuellen(4.5, True);
      end;
      12:
      begin
        Info('Starte Wasserweichen-Automatik (alle 70 Sekunden wechseln)');
        PrepareNextStep(10, True);
        Main.Form1.AutoWWStart(70000);
      end;
      13:
      begin
        Info('Umwlzen starten');
        PrepareNextStep(500, True);
        Main.Form1.Machine.Machine_Umwaelzen(True);
      end;
      14:
      begin
        Info('Reiniger zugeben');
        Main.Form1.ReinigerOeffnen(True);
      end;
      15:
      begin
        Info('Heizen 65, Umwlzen, warten bis warm');
        Main.Form1.HeizenStart(65, True, True);
      end;
      16:
      begin
        Info('Warte weitere 120 Sekunden');
        PrepareNextStep(120000, True);
      end;
      17:
      begin
        Info('Heizen Stop, weiter umwlzen');
        PrepareNextStep(500, True);
        Main.Form1.HeizenStop;
        Main.Form1.Machine.Machine_Umwaelzen(True);
      end;
      18:
      begin
        Info('Fllen 1 Liter');
        PrepareNextStep(10, False);
        Main.Form1.Fuellen(1, True);
      end;
      19:
      begin
        Info('Warte weitere 120 Sekunden');
        PrepareNextStep(120000, True);
      end;
      20:
      begin
        Info('Wasserweichen-Automatik & Umwlzen Stop');
        Main.Form1.AutoWWStop;
        Main.Form1.Machine.Machine_Umwaelzen(False);
        PrepareNextStep(5000, True);
      end;
      21:
      begin
        Info('Wasser abpumpen, Reinigen fertig');
        PrepareNextStep(10, False);
        Main.Form1.Ausleeren(0, True);
      end;

      22:
      begin
        Info('Wasserweiche einstellen: OK');
        PrepareNextStep(10, True);
        Main.Form1.Korbwahl(2, False);
      end;


      23:
      begin
        Info('### Zwischensplen ###');
        Info('Fllen 4 Liter');
        PrepareNextStep(10, False);
        Main.Form1.Fuellen(4, True);
      end;
      24:
      begin
        Info('Starte Wasserweichen-Automatik (alle 70 Sekunden wechseln)');
        PrepareNextStep(10, True);
        Main.Form1.AutoWWStart(70000);
      end;
      25:
      begin
        Info('Umwlzen starten');
        PrepareNextStep(500, True);
        Main.Form1.Machine.Machine_Umwaelzen(True);
      end;
      26:
      begin
        Info('Warte 60 Sekunden auf Temperatur-Sensor');
        PrepareNextStep(120000, True);
      end; 
      27:
      begin
        Info('Heizen 55, Umwlzen, warten bis warm');
        Main.Form1.HeizenStart(55, True, True);
      end;
      28:
      begin
        Info('Warte weitere 120 Sekunden');
        PrepareNextStep(120000, True);
      end;
      29:
      begin
        Info('Heizen, Wasserweichen-Automatik & Umwlzen Stop');
        Main.Form1.HeizenStop;
        Main.Form1.AutoWWStop;
        Main.Form1.Machine.Machine_Umwaelzen(False);
        PrepareNextStep(5000, True);
      end;
      30:
      begin
        Info('Wasser abpumpen, Zwischensplen fertig');
        PrepareNextStep(10, False);
        Main.Form1.Ausleeren(0, True);
      end;

      31:
      begin
        Info('Wasserweiche einstellen: OK');
        PrepareNextStep(10, True);
        Main.Form1.Korbwahl(2, False);
      end;


      32:
      begin
        Info('### Klarsplen ###');
        Info('Fllen 4 Liter');
        PrepareNextStep(10, False);
        Main.Form1.Fuellen(4, True);
      end;
      33:
      begin
        Info('Starte Wasserweichen-Automatik (alle 70 Sekunden wechseln)');
        PrepareNextStep(10, True);
        Main.Form1.AutoWWStart(70000);
      end;
      34:
      begin
        Info('Heizen 60, Umwlzen, warten bis warm');
        Main.Form1.HeizenStart(60, True, True);
      end;
      35:
      begin
        Info('Warte weitere 120 Sekunden');
        PrepareNextStep(120000, True);
      end;
      36:
      begin
        Info('Heizen Stop, weiter umwlzen');
        PrepareNextStep(2000, True);
        Main.Form1.HeizenStop;
        Main.Form1.Machine.Machine_Umwaelzen(True);
      end;
      37:
      begin
        Info('### Trocknen ###');
        Info('Fllen 0,5 l');
        Main.Form1.Fuellen(0.5, True);
      end;
      38:
      begin
        Info('Heizen 65, Umwlzen, warten bis warm');
        Main.Form1.HeizenStart(65, True, True);
      end;
      39:
      begin
        Info('Warte weitere 60 Sekunden');
        PrepareNextStep(60000, True);
      end;
      40:
      begin
        Info('Heizen, Wasserweichen-Automatik & Umwlzen Stop');
        Main.Form1.HeizenStop;
        Main.Form1.AutoWWStop;
        Main.Form1.Machine.Machine_Umwaelzen(False);
        PrepareNextStep(10, True);
      end;
      41:
      begin
        Info('Warte weitere 30 Sekunden');
        PrepareNextStep(30000, True);
      end;
      42:
      begin
        Info('Wasser abpumpen');
        PrepareNextStep(10, False);
        Main.Form1.Ausleeren(0, True);
      end;
      43:
      begin
        Info('Fllen 3 Liter');
        PrepareNextStep(10, False);
        Main.Form1.Fuellen(3, True);
      end;
      44:
      begin
        Info('Warte weitere 120 Sekunden');
        PrepareNextStep(120000, True);
      end;
      45:
      begin
        Info('Wasser abpumpen, trocknen fertig');
        PrepareNextStep(500, False);
        Main.Form1.Ausleeren(28000, True);
      end;
      46:
      begin
        Info('Programm fertig');
        PrepareNextStep(10, True);
      end;
    end;

    if CurrentStep > ProgrammSteps then
    begin
      ProgrammRunning := False;
      Main.Form1.ProgrammText3.Caption := 'Fertig, hoffentlich sauber!';
      Main.Form1.AddTipEntry('Programm durchgelaufen');
      Info('Programm durchgelaufen');
      Main.Form1.StartFertigAlarm;
    end;

  end;
end;

{$ENDREGION}

////////////////////////////////////////////////////////////////////////////////

end.
